@@ -32,15 +32,16 @@
 
 #include "config.h"
 #include "download.h"
+#include "filemap.h"
 #include "hash.h"
 #include "help_helper.h"
 #include "logging.h"
 #include "outfmt.h"
 #include "placeholders.h"
 #include "savefile_aux.h"
-#include "text.h"
 #include "str.h"
 #include "subcmd_common.h"
+#include "text.h"
 
 typedef struct
 {
@@ -131,7 +132,7 @@ static int str_to_ushort(const char *str, unsigned short *out)
     else if (*endptr != '\0')
         warnx("invalid short value: '%s'", str);
     else if (result > USHRT_MAX)
-        warnx("value too big: %lu, max is %hu", result, USHRT_MAX);
+        warnx("value too big: %lu, max is %hu", result, (unsigned short)USHRT_MAX);
     else {
         *out = result;
         return 0;
@@ -253,13 +254,13 @@ static void rss_show_error(const fetch_ctx_t *fctx, mrss_error_t e)
           STR_FMT(&url), e, kind, reason);
 }
 
-static mrss_t *rss_open(const fetch_ctx_t *fctx, const char *file_path)
+static mrss_t *rss_open(const fetch_ctx_t *fctx, const filemap_t *filemap)
 {
     mrss_t *rss;
     mrss_error_t error;
 
     /* mrss_parse_file requires `char *`.  Old style software, I guess. */
-    error = mrss_parse_file((char *)file_path, &rss);
+    error = mrss_parse_buffer(filemap->memory, filemap->size, &rss);
     if (!error)
         return rss;
 
@@ -406,7 +407,7 @@ exit:
     gctx->ofmt_loaded = ofmt_loaded;
 }
 
-static int feed_fetch_local(fetch_ctx_t *fctx, const char *file_path)
+static int feed_fetch_local(fetch_ctx_t *fctx, const filemap_t *filemap)
 {
     enum {
         critical,
@@ -421,7 +422,7 @@ static int feed_fetch_local(fetch_ctx_t *fctx, const char *file_path)
     };
 
     say("feed [%s]", feed_get_name(fctx->feed));
-    mrss_t *mrss = rss_open(fctx, file_path);
+    mrss_t *mrss = rss_open(fctx, filemap);
     if (!mrss) {
         status = feed_failure;
         goto exit;
@@ -507,23 +508,26 @@ exit:
 
 static int on_downloaded(void *opaque,
                          download_status_t status,
-                         const char *filename)
+                         const struct filemap_t *filemap)
 {
     fetch_ctx_t *fctx = opaque;
     int err = 0;
 
     switch (status) {
     case dl_complete:
-        err = feed_fetch_local(fctx, filename);
+        err = feed_fetch_local(fctx, filemap);
         break;
 
-    case dl_failed:
+    case dl_sys_failure:
+        err = -1; /* system failures will also terminate the download */
+    case dl_fetch_failure:
+        warnx("failed download of [%s]", feed_get_name(fctx->feed));
         fctx->gctx->counters.failed_feeds++;
         fetch_ctx_free(fctx);
         break;
 
     case dl_aborted:
-        say("feed download aborted (%s)", fctx->url);
+        say("aborted download of [%s]", fctx->url);
         fetch_ctx_free(fctx);
         break;
 
@@ -560,11 +564,18 @@ static int feed_fetch(global_ctx_t *gctx, feed_t *feed)
     case feed_ut_local:
         {
             fetch_ctx_t fctx;
+            int e;
+            filemap_t filemap;
 
             if (!fetch_ctx_init(&fctx, gctx, feed))
                 return -1;
 
-            return feed_fetch_local(&fctx, fctx.url);
+            if (filemap_load_path(&filemap, fctx.url))
+                return -1;
+
+            e = feed_fetch_local(&fctx, &filemap);
+            filemap_unload(&filemap);
+            return e;
         }
 
     case feed_ut_unknown:
