@@ -26,10 +26,13 @@ typedef struct download download_t;
 
 typedef enum {
     dl_complete,
-    dl_failed,
     dl_aborted,
+    dl_fetch_failure,
+    dl_sys_failure,
 } download_status_t;
 
+struct filemap_t;
+
 /* download_cb_t - download callback.
  *
  * Expected to return 0 on success.  If the return value is not 0 all
@@ -38,7 +41,7 @@ typedef enum {
 typedef int (*download_cb_t)(
     void *opaque,
     download_status_t,
-    const char *filename
+    const struct filemap_t *filemap
 );
 
 download_t *download_new(bool global_init, unsigned maxjobs);
