@@ -115,7 +115,7 @@ int placeholders_setup(ofmt_t ofmt)
 
     state = malloc(sizeof(struct state) + buflen);
     if (!state) {
-        warn("malloc failed");
+        warn("malloc");
         return -1;
     }
     *state = (state_t){ .buflen = buflen };
