@@ -18,6 +18,7 @@
  * <https://www.gnu.org/licenses/>.
  */
 
+#include "filemap.h"
 #include "logging.h"
 #include "savefile.h"
 #include "str.h"
@@ -49,7 +50,6 @@ typedef enum file_status_t file_status_t;
 typedef enum flag_idx_t flag_idx_t;
 
 typedef struct revision_t revision_t;
-typedef struct filemap_t filemap_t;
 typedef struct err_t err_t;
 typedef struct feed_t feed_t;
 typedef struct entry_t entry_t;
@@ -91,12 +91,6 @@ struct revision_t
     uint8_t minor;
 } __attribute__((packed, aligned(1)));
 
-struct filemap_t
-{
-    void *memory;
-    size_t size;
-};
-
 struct err_t
 {
     /*  >= 0    errno_val = -value  error = sfe_system_error
@@ -152,10 +146,6 @@ const char *file_status_str[] = {
 
 static const char *dotstrdup(const char *);
 static int writev_wrap(int fd, struct iovec *, size_t);
-static int filesize(int fd, size_t *s);
-
-static int filemap_load(filemap_t *, int fd);
-static void filemap_unload(filemap_t *);
 
 static err_t err_set(int enum_value);
 static savefile_err_t err_get(err_t);
@@ -223,58 +213,6 @@ static int writev_wrap(int fd, struct iovec *iovec, size_t size)
     return -1;
 }
 
-static int filesize(int fd, size_t *s)
-{
-    struct stat sb;
-
-    if (fstat(fd, &sb) == -1) {
-        warn("fstat failed");
-        return -1;
-    }
-
-    *s = sb.st_size;
-    return 0;
-}
-
-static int filemap_load(filemap_t *fmap, int fd)
-{
-    size_t size;
-    void *mapped;
-
-    if (filesize(fd, &size) == -1)
-        return -1;
-
-    *fmap = (filemap_t) {};
-    if (size == 0)
-        /* An empty file is not an error.  It is handled as there
-         * was no file in the first place */
-        goto success;
-
-    mapped = mmap(NULL, size, PROT_READ, MAP_PRIVATE, fd, 0);
-    if (mapped == MAP_FAILED) {
-        warn("mmap failed");
-        return -1;
-    }
-
-    *fmap = (filemap_t) {
-        .memory = mapped,
-        .size = size,
-    };
-
-success:
-    close(fd);
-    return 0;
-}
-
-static void filemap_unload(filemap_t *fmap)
-{
-    if (fmap->memory == MAP_FAILED || !fmap->memory)
-        return;
-
-    munmap(fmap->memory, fmap->size);
-    *fmap = (filemap_t) {};
-}
-
 static err_t err_set(int enum_value)
 {
     return (err_t){
@@ -481,7 +419,7 @@ static int feed_load(feed_t *feed, int fd)
      * 'file_status' field will be set accordingly.
      */
 
-    if (filemap_load(&feed->filemap, fd) == -1)
+    if (filemap_load_fd(&feed->filemap, fd) == -1)
         return -1;
 
     int read_offset = feed_load_magic(feed);
